
% this program shows how to get a "smooth spline" interpolation curve of a
% jagged piecewise linear curve.

clear all; close all;

% Just for illustration, we define 
xlist = linspace(0,1,10)';
flist = [1; 2; 1; 1.5; 2; 2; 3; 2.7; 2.5; 2.3];

figure(1);
plot(xlist,flist,'b.-'); xlabel('x'); ylabel('f');
title('Some piecewise linear curve');
axis([0 1 0 3]);

% Now we define a curve on many many more x-points. The f values are
% obtained by 'spline' interpolation, which here means that a piecewise
% cubic curve is fit between the grid points, so that the whole curve is
% continuous and has continuous first and second derivatives.

xlist_dense = linspace(0,1,1000)';
flist_dense = interp1(xlist,flist,xlist_dense,'spline','extrap');
figure(2); plot(xlist,flist,'b.-'); xlabel('x'); ylabel('f'); hold on;
plot(xlist_dense,flist_dense,'r');
title('Normal Spline Interpolation (this is the most common meaning of "spline")');
axis([0 1 0 3]);


% a different cubic interpolation that is "shape-preserving".

xlist_dense = linspace(0,1,1000)';
flist_dense = interp1(xlist,flist,xlist_dense,'pchip','extrap');

figure(3);
plot(xlist,flist,'b.-');  hold on;
plot(xlist_dense,flist_dense,'r');
xlabel('x'); ylabel('f');
title('Shape preserving spline interpolation (that is, splines without overshoots)')
axis([0 1 0 3]);
